def get_swagger_config():
    return {
        "swagger": "2.0",
        "info": {
            "title": "OrKa Vector API",
            'description': 'The API documentation for the OrKa Vector API',
        },
        "schemes": [
            "http",
            "https"
        ]
    }
